//-------------------------------------------------
// Routines lecture encodeurs affichage des menus
// pour le programme OneBitSynth
//-------------------------------------------------

//----------------------------------
// Affiche logo defilement des menus
// Si le flag changement est actif
//----------------------------------
void ChangeMenuMode(void) {
	if (!changemenu)
		return;
	lcd.setCursor(17,0);		
	lcd.printByte(CAR_LEFT);
	lcd.print(F("M"));
	lcd.printByte(CAR_RIGHT);	// Affichage du flag <--> pour montrer que l'on passe en defilement
}

//----------------------------------------------------
// Affiche la fleche de selection de parametre
// uniquement si on est pas en mode defilement de menu
//----------------------------------------------------
void AffSelectParam(void) {
	if (!changemenu) {			// Si mode edition
		lcd.printByte(CAR_TRIRIGHT);	// On affiche la selection du premier parametre
	}
	else {
		 lcd.print(F(" "));			// Sinon on affiche un espace
	}
}

//---------------------------------------		
// Initialisation d'un menu
// en fonction de la variable nummenu
// qui contient le numero de menu courant
//---------------------------------------		
void InitMenu() {
	lcd.clear();		// Effacement LCD
	posmenu=0;			// Reinitialise la selection dans le menu au cas ou...
	switch(nummenu) {
		// Menu edition oscillateur
		case 0:
			lcd.print(F("EDIT>Oscillator"));
			ChangeMenuMode();				// Affiche mode defilement de menu si besoin
			lcd.setCursor(0,1);
			lcd.print(F("Wave:"));
			AffSelectParam();				// Affiche la selection du parametre si necessaire
			digitsLcd(wavenum,2);
			lcd.setCursor(0,2);
			lcd.print(F("Freq: "));
			lcd.setCursor(0,3);
			lcd.print(F("Midi: "));
			RaffRefreshMidiModeAndPitch();
			break;
		// Menu edition LFO
		case 1:
			lcd.print(F("EDIT>Lfo"));
			ChangeMenuMode();				// Affiche mode defilement de menu si besoin
			lcd.setCursor(1,1);
			lcd.print(F("Wave:"));
			AffSelectParam();				// Affiche la selection du parametre si necessaire
			affOndesLcdPROGMEM(lfowavenum);
			lcd.setCursor(1,2);
			lcd.print(F("Freq: "));
			digitsLcd(lfofreq,3);
			lcd.setCursor(0,3);
			lcd.print(F("Depth: "));
			digitsLcd(lfodepth,3);
			break;
		// Menu edition Enveloppe
		case 2:
			lcd.print(F("EDIT>EnvGen"));
			ChangeMenuMode();				// Affiche mode defilement de menu si besoin
			lcd.setCursor(1,2);
			lcd.print(F("Freq:"));
			AffSelectParam();				// Affiche la selection du parametre si necessaire
			digitsLcd(envfreq,3);
			lcd.setCursor(0,3);
			lcd.print(F("Width: "));
			digitsLcd(envwidth,3);
			break;
		// Menu edition Refresh rate
		case 3:
			lcd.print(F("EDIT>Rate"));
			ChangeMenuMode();				// Affiche mode defilement de menu si besoin
			lcd.setCursor(0,2);
			lcd.print(F("Lfo:"));
			AffSelectParam();				// Affiche la selection du parametre si necessaire
			digitsLcd(RefreshRateLfo,2);
			lcd.print(F(" ms"));
			lcd.setCursor(0,3);
			lcd.print(F("Env: "));
			digitsLcd(RefreshRateEnv,2);
			lcd.print(F(" ms"));
			break;	
		// Menu edition Userwave
		case 4:
			lcd.print(F("EDIT>User Wave"));
			ChangeMenuMode();				// Affiche mode defilement de menu si besoin
			lcd.setCursor(0,2);
			lcd.printByte(CAR_PLAIN);
			lcd.printByte(CAR_PLAIN);
			aff_binary_integer(UserWaveform);	// Affiche la forme onde utilisateur en binaire
			lcd.printByte(CAR_PLAIN);
			lcd.printByte(CAR_PLAIN);
			if (!changemenu) {					// Si en dehors du mode defilement de menu
				PosUserWaveform=1;				// Remettre le pointeur de position sur 1
				lcd.setCursor(2,1);				// Mettre les curseurs sur le bit de poids fort
				lcd.printByte(CAR_TRIDOWN);		
				lcd.setCursor(2,3);
				lcd.printByte(CAR_TRIUP);		
			}
			break;	
			
		
		// Menu Presets
		case 10:
			lcd.print(F("PRESETS>Select"));
			lcd.setCursor(0,2);
			lcd.print(F("LOAD:"));
			lcd.printByte(CAR_TRIRIGHT);
			digitsLcd(NumPresetLoad,2);
			lcd.setCursor(0,3);
			lcd.print(F("SAVE: "));
			digitsLcd(NumPresetSave,2);
			break;	
		}
		
}

//-------------------------------------------------
// Lecture des boutons des encoders
// mets à jour les menus et reaffiche si nécessaire
//-------------------------------------------------
void UpdateParamButtons() {
	valbouton=readButtons();
	// if (!valbouton)			// Si aucun bouton appuye
		// return;				// sortir
	switch(valbouton) {
		// si bouton menu
		// on revient au debut
		case B_ENC1:
			// On verifie d'abord qu'on est pas dans le second menu des presets
			if (nummenu>MAXMENU1) {	// Si nous sommes dans le menu des presets
				posmenu=0;			// position parametre 1 dans le menu
				nummenu=MINMENU1;	// position premier menu edition
				changemenu=0;		// Normalement a zero mais on le force quand meme !!!
				InitMenu();			// affichage du menu
				return;
			}

			// Si on arrive ici c'est que nous somme déja dans les menus editions du synthe
			// Si on est en edition de parametre on bascule en mode changement de menu
			// Si on est en changement de menu, on repasse en edition de parametre
			if(!changemenu) {		// Si nous etions en edition de parametre
				changemenu=1;		// On passe en mode defilement des menus
				InitMenu();			// Rafraichissement du menu
				return;
			}
			else {					// Sinon on était on mode defilement de menu
				changemenu=0;		// On repasse en mode edition de parametre
				InitMenu();			// Rafraichissement du menu
				return;
			}
			break;
		case B_ENC2:
			changemenu=0;			// On quitte le mode defilement de menu si il etait actif
			// Teste si on est sur menu encodeur 1
			if (nummenu<MINMENU2) {
				nummenu=MINMENU2;	// si oui revenir au premier menu encodeur 2
				posmenu=0;			// activer le premier element du menu actif
				InitMenu();
			}
			else {					// sinon on est deja dans les menu de l'encodeur 2
				switch(posmenu) {
					case 0:
						LoadPreset(NumPresetLoad,1);	// Loader le preset selectionne
						break;
					case 1:
						SavePreset(NumPresetSave,1);	// Sauver le preset selectionne
						break;
				}
			}
			return;
			break;
	}
}

// Rafraichit forme onde oscillateur sur LCD
void RaffLcdOscWave(void) {
	if (nummenu!=MENUOSC)
		return;
	digitsLcdPos(wavenum,6,1,2);

}
// Rafraichit valeur pitch oscillateur sur LCD
void RaffLcdOscPitch(void) {
	digitsLcdPos(pitch,6,2,4);
}
// Rafraichit affichage du mode midi (on ou off)
void RaffRefreshMidiMode(void) {
//	OnOffLcd(usemidi,6,3);
	lcd.setCursor(6,3);
	switch(usemidi) {
		case 0:
			lcd.print(F("Off    "));
			break;
		case 1:
			lcd.print(F("On     "));
			break;
		case 2:
			lcd.print(F("CV/Gate"));
			break;
	}
}


// Rafraichit affichage du mode midi (on ou off)
void RaffRefreshMidiModeAndPitch(void) {
	//OnOffLcd(usemidi,6,3);
	RaffRefreshMidiMode();
	if (usemidi) {
		lcd.setCursor(6,2);
		lcd.print(F("----"));
	}
	else {
		digitsLcdPos(pitch,6,2,4);
	}
	
}

// Rafraichit forme onde LFO
void RaffLcdLfoWave(void) {
	lcd.setCursor(7,1);
//	affOndesLcdPROGMEM(lfowavenumButton);
	affOndesLcdPROGMEM(lfowavenum);
}
// Rafraichit forme onde LFO
void RaffLcdLfoFreq(void) {
	digitsLcdPos(lfofreq,7,2,3);
}
// Rafraichit profondeur du LFO
void RaffLcdLfoDepth(void) {
	digitsLcdPos(lfodepth,7,3,3);
}
// Rafraichit frequence env. generator
void RaffLcdEnvFreq(void) {
	digitsLcdPos(envfreq,7,2,3);
}
// Rafraichit profondeur env. generator
void RaffLcdEnvWidth(void) {
	digitsLcdPos(envwidth,7,3,3);
}
void RaffRefreshRateLfo(void) {
	digitsLcdPos(RefreshRateLfo,5,2,2);
}

void RaffRefreshRateEnv(void) {
	digitsLcdPos(RefreshRateEnv,5,3,2);
}
void RaffNumPresetLoad(void) {
	digitsLcdPos(NumPresetLoad,6,2,2);
}

void RaffNumPresetSave(void) {
	digitsLcdPos(NumPresetSave,6,3,2);
}

//------------------------------------------
// Gestion defilement des menus
//------------------------------------------
void DefilementMenu(void) {
	switch(valencodeurs) {
		case ENC1UP:
			if (nummenu<MAXMENU1) {	// Si dernier menu non atteint
				nummenu++;			// menu suivant
				InitMenu();			// affichage du menu
			}		
			return;
			break;
		case ENC1DOWN:
			if (nummenu>MINMENU1) {	// Si nous sommes au dela du premier menu
				nummenu--;			// menu precedent
				InitMenu();			// affichage du menu
			}		
			return;
			break;

		// case ENC2UP:
			// break;
		// case ENC2DOWN:
			// break;
	}
} 


//------------------------------------------
// Gestion menu oscillateur
// gere la selection de parametre
// et le changement de valeur des parametres
//------------------------------------------
void OscillatorMenu(void) {
	switch(valencodeurs) {
		case ENC1UP:
			if (posmenu>=2)
				return;
			lcd.setCursor(5,posmenu+1);
			lcd.print(F(" "));
			posmenu++;
			lcd.setCursor(5,posmenu+1);
			lcd.printByte(CAR_TRIRIGHT);
			break;
		case ENC1DOWN:
			if (!posmenu)
				return;
			lcd.setCursor(5,posmenu+1);
			lcd.print(F(" "));	
			posmenu--;
			lcd.setCursor(5,posmenu+1);
			lcd.printByte(CAR_TRIRIGHT);
			break;
		case ENC2UP:
			switch(posmenu) {
				// Increment forme onde Oscillator
				case 0:	
					if (wavenum>=MAXWAVESOSC)
						return;
					wavenum++;
					if (wavenum!=8)
						waveform = waveforms[wavenum];
					else
						waveform = UserWaveform;
					RaffLcdOscWave();
					break;
				// Increment pitch oscillator
				case 1:
					if (pitch>=MAXPITCHOSC)
						return;
						if (usemidi)
							return;
					if (pitch<200)
						pitch++;
					if (pitch>=200 && pitch<500)
						pitch+=2;
					if (pitch>=500)
						pitch+=5;
					if (pitch>=1000)
						pitch+=10;
					if (pitch>=MAXPITCHOSC)
						pitch=MAXPITCHOSC;
					RaffLcdOscPitch();
					break;
				case 2:
					if (usemidi<2) {
						usemidi++;
						RaffRefreshMidiModeAndPitch();
						noteon=0;
					}
					break;
			}
			break;
		case ENC2DOWN:
			switch(posmenu) {
				// Decrement forme onde Oscillator
				case 0:	
					if (!wavenum)
						return;
					wavenum--;
					if (wavenum!=8)
						waveform = waveforms[wavenum];
					else
						waveform = UserWaveform;
					RaffLcdOscWave();
					break;
				// Decrement pitch oscillator
				case 1:
					if (pitch>32000)
						return;
					if (usemidi)
						return;
					if (pitch<200)
						pitch--;
					if (pitch>=200 && pitch<500)
						pitch-=2;
					if (pitch>=500 && pitch<1000)
						pitch-=5;
					if (pitch>=1000)
						pitch-=10;
						
					if (pitch>32000)
						pitch=0;
						
					RaffLcdOscPitch();
					break;
				case 2:
					if (usemidi) {
						usemidi--;
						if (!usemidi)
							pitch=100;
						RaffRefreshMidiModeAndPitch();
						noteon=1;
					}
					break;
			}
			break;
	}
} 
//------------------------------------------
// Gestion menu LFO
// gere la selection de parametre
// et le changement de valeur des parametres
//------------------------------------------
void LfoMenu(void) {
	switch(valencodeurs) {
		case ENC1UP:
			if (posmenu>=2)
				return;
			lcd.setCursor(6,posmenu+1);
			lcd.print(F(" "));
			posmenu++;
			lcd.setCursor(6,posmenu+1);
			lcd.printByte(CAR_TRIRIGHT);
			break;
		case ENC1DOWN:
			if (!posmenu)
				return;
			lcd.setCursor(6,posmenu+1);
			lcd.print(F(" "));	
			posmenu--;
			lcd.setCursor(6,posmenu+1);
			lcd.printByte(CAR_TRIRIGHT);
			break;
		case ENC2UP:
			switch(posmenu) {
				// Increment forme onde LFO
				case 0:	
					if (lfowavenum>=MAXWAVESLFO)
						return;
					lfowavenum++;
					RaffLcdLfoWave();
					break;
				// Increment frequence lfo
				case 1:
					if (lfofreq>=MAXLFOFREQ)
						return;
					lfofreq++;
					RaffLcdLfoFreq();
					break;
				// Increment profondeur lfo
				case 2:
					if (lfodepth>=MAXLFODEPTH)
						return;
					lfodepth++;
					RaffLcdLfoDepth();
					break;
			}
			break;
		case ENC2DOWN:
			switch(posmenu) {
				// Increment forme onde LFO
				case 0:	
					if (!lfowavenum)
						return;
					lfowavenum--;
					RaffLcdLfoWave();
					break;
				// Increment frequence lfo
				case 1:
					if (!lfofreq)
						return;
					lfofreq--;
					RaffLcdLfoFreq();
					break;
				// Increment profondeur lfo
				case 2:
					if (!lfodepth)
						return;
					lfodepth--;
					RaffLcdLfoDepth();
					break;
			}
			break;
	}
} 
//------------------------------------------
// Gestion menu Enveloppe
// gere la selection de parametre
// et le changement de valeur des parametres
//------------------------------------------
void EnvelopeMenu(void) {
	switch(valencodeurs) {
		case ENC1UP:
			if (posmenu>=1)
				return;
			lcd.setCursor(6,posmenu+2);
			lcd.print(F(" "));
			posmenu++;
			lcd.setCursor(6,posmenu+2);
			lcd.printByte(CAR_TRIRIGHT);
			break;
		case ENC1DOWN:
			if (!posmenu)
				return;
			lcd.setCursor(6,posmenu+2);
			lcd.print(F(" "));	
			posmenu--;
			lcd.setCursor(6,posmenu+2);
			lcd.printByte(CAR_TRIRIGHT);
			break;
		case ENC2UP:
			switch(posmenu) {
				// Increment freq env. generator
				case 0:	
					if (envfreq>=MAXENVFREQ)
						return;
					envfreq++;
					RaffLcdEnvFreq();
					break;
				// Increment largeur env. generator
				case 1:
					if (envwidth>=MAXENVWIDTH)
						return;
					envwidth++;
					RaffLcdEnvWidth();
					break;
			}
			break;
		case ENC2DOWN:
			switch(posmenu) {
				// Decrement frequence env. generator
				case 0:	
					if (!envfreq)
						return;
					envfreq--;
					RaffLcdEnvFreq();
					break;
				// Decrement largeur env. generator
				case 1:
					if (!envwidth)
						return;
					envwidth--;
					RaffLcdEnvWidth();
					break;
			}
			break;
	}
} 

//------------------------------------------
// Gestion menu Enveloppe
// gere la selection de parametre
// et le changement de valeur des parametres
//------------------------------------------
void RefreshRateMenu(void) {
	switch(valencodeurs) {
		case ENC1UP:
			if (posmenu>=1)
				return;
			lcd.setCursor(4,posmenu+2);
			lcd.print(F(" "));
			posmenu++;
			lcd.setCursor(4,posmenu+2);
			lcd.printByte(CAR_TRIRIGHT);
			break;
		case ENC1DOWN:
			if (!posmenu)
				return;
			lcd.setCursor(4,posmenu+2);
			lcd.print(F(" "));	
			posmenu--;
			lcd.setCursor(4,posmenu+2);
			lcd.printByte(CAR_TRIRIGHT);
			break;
		case ENC2UP:
			switch(posmenu) {
				// Increment delai rafraichissement LFO
				case 0:	
					if (RefreshRateLfo>=MAX_REFRESH_RATE_LFO)
						return;
					RefreshRateLfo++;
					RaffRefreshRateLfo();
					break;
				// Increment delai rafraichissement Enveloppe
				case 1:
					if (RefreshRateEnv>=MAX_REFRESH_RATE_ENV)
						return;
					RefreshRateEnv++;
					RaffRefreshRateEnv();
					break;
			}
			break;
		case ENC2DOWN:
			switch(posmenu) {
				// Decrement delai rafraichissement LFO
				case 0:	
					if (RefreshRateLfo<=MIN_REFRESH_RATE_LFO)
						return;
					RefreshRateLfo--;
					RaffRefreshRateLfo();
					break;
				// Decrement delai rafraichissement Enveloppe
				case 1:
					if (RefreshRateEnv<=MIN_REFRESH_RATE_ENV)
						return;
					RefreshRateEnv--;
					RaffRefreshRateEnv();
					break;
			}
			break;
	}
} 

//------------------------------------------
// Gestion menu des presets
// permet de selectionner un preset 
// et de le charger ou de le sauvegarder
//------------------------------------------
void PresetMenu(void) {
	switch(valencodeurs) {
		case ENC1UP:
			if (posmenu>=1)
				return;
			lcd.setCursor(5,posmenu+2);
			lcd.print(F(" "));
			posmenu++;
			lcd.setCursor(5,posmenu+2);
			lcd.printByte(CAR_TRIRIGHT);
			break;
		case ENC1DOWN:
			if (!posmenu)
				return;
			lcd.setCursor(5,posmenu+2);
			lcd.print(F(" "));	
			posmenu--;
			lcd.setCursor(5,posmenu+2);
			lcd.printByte(CAR_TRIRIGHT);
			break;
		case ENC2UP:
			switch(posmenu) {
				// Increment numero preset a loader
				case 0:	
					if (NumPresetLoad>=MAXPRESETS)
						return;
					NumPresetLoad++;
					RaffNumPresetLoad();
					break;
				// Increment numero de preset a sauvergarder
				case 1:
					if (NumPresetSave>=MAXPRESETS)
						return;
					NumPresetSave++;
					RaffNumPresetSave();
					break;
			}
			break;
		case ENC2DOWN:
			switch(posmenu) {
				// Decrement numero de preset a loader
				case 0:	
					if (NumPresetLoad<=1)
						return;
					NumPresetLoad--;
					RaffNumPresetLoad();
					break;
				// Decrement numero de preset a sauvegarder
				case 1:
					if (NumPresetSave<=1)
						return;
					NumPresetSave--;
					RaffNumPresetSave();
					break;
			}
			break;
	}
} 

//------------------------------------------
// Gestion menu edition userWave
// permet de changer la valeur de chaque bit
// de la forme d'onde de 16 bits
//------------------------------------------
void UserWaveMenu(void) {
	switch(valencodeurs) {
		case ENC1UP:
			if (PosUserWaveform>=MAXPOSWAVEFORMUSER)
				return;
			lcd.setCursor(1+PosUserWaveform,1);			// Positionner le curseur
			lcd.print(F(" "));							// effacer pointeur de position
			lcd.setCursor(1+PosUserWaveform,3);			// Positionner le curseur
			lcd.print(F(" "));							// effacer pointeur de position
			PosUserWaveform++;							// Incrementer le pointeur de position
			lcd.setCursor(1+PosUserWaveform,3);			// Positionner le curseur
			lcd.printByte(CAR_TRIUP);
			lcd.setCursor(1+PosUserWaveform,1);			// Positionner le curseur
			lcd.printByte(CAR_TRIDOWN);
			
			break;
		case ENC1DOWN:
			if (PosUserWaveform==MINPOSWAVEFORMUSER)
				return;
			lcd.setCursor(1+PosUserWaveform,1);			// Positionner le curseur
			lcd.print(F(" "));							// effacer pointeur de position
			lcd.setCursor(1+PosUserWaveform,3);			// Positionner le curseur
			lcd.print(F(" "));							// effacer pointeur de position
			PosUserWaveform--;							// Incrementer le pointeur de position
			lcd.setCursor(1+PosUserWaveform,3);			// Positionner le curseur
			lcd.printByte(CAR_TRIUP);
			lcd.setCursor(1+PosUserWaveform,1);			// Positionner le curseur
			lcd.printByte(CAR_TRIDOWN);
			break;
		case ENC2UP:
			if (bitRead(UserWaveform,16-PosUserWaveform))	// Si le bit en cours est deja a 1
				return;										// on sort
			// Arrive ici le bit est forcement a 0 et on le passe a 1
			bitSet(UserWaveform,16-PosUserWaveform);
			lcd.setCursor(1+PosUserWaveform,2);			// Positionner le curseur
			lcd.print(F("1"));
			ReloadUserWaveform();						// recharger la forme d'onde si besoin
			break;
		case ENC2DOWN:
			if (!bitRead(UserWaveform,16-PosUserWaveform))	// Si le bit en cours est deja a 0
				return;										// on sort
			// Arrive ici le bit est forcement a 1 et on le passe a 0
			bitClear(UserWaveform,16-PosUserWaveform);		// mettre a zero le bit de ka
			lcd.setCursor(1+PosUserWaveform,2);			// Positionner le curseur
			lcd.print(F("0"));
			ReloadUserWaveform();						// recharger la forme d'onde si besoin
			break;
	}
} 


//--------------------------------------------
// Procedure generale de lecture des encodeurs
//--------------------------------------------
void UpdateParamEncoders() {
	
	valencodeurs=readEncodeurs();	// Lecture des encodeurs
	if (!valencodeurs)				// Si aucun encodeur n'a bouge
		return;						// On sort, inutile de faire les tests suivants
	
	// Arrive ici un encodeur a subit une modification de position
	if (changemenu) {				// si mode defilement de menu
		DefilementMenu();			// On lance procedure de gestion de defilement
		return;
	}
	
	// Arrive ici on est en gestion de parametre au sein d'un menu
	// Lancement de la procedure de menu correcte
	// en fonction du menu courant
	switch(nummenu) {
		case 0:
			OscillatorMenu();
			break;
		case 1:
			LfoMenu();
			break;
		case 2:
			EnvelopeMenu();
			break;
		case 3:
			RefreshRateMenu();
			break;
		case 4:
			UserWaveMenu();
			break;
		case 10:
			PresetMenu();
			break;
			
	}
}

//---------------------------------------------
// Mise a jour de tous les parametres du synthe
// Lecture des encodeurs
// lectures des boutons
// Lecture des infos extérieures (CV, GATE...)
//---------------------------------------------
void RefreshParamMenu() {
	UpdateParamButtons();	// Lecture des boutons
	UpdateParamEncoders();	// Lecture des encodeurs
	// UpdateParamCVGate();
}

