//---------------------------------------------------------------
// PROGRAMME OneBitSynth.ino
// Synthetiseur 1 bit
// Adaptation du programme de Matt Sarnoff
//---------------------------------------------------------------
// http://www.msarnoff.org/1bitsynth/
// 1-Bit AVR Synthesizer
// By Matt Sarnoff (msarnoff.org/1bitsynth)
// October 2009-October 2010
//---------------------------------------------------------------
// pour une utilisation sur ATMEGA328P avec des encodeurs et un LCD
// a la place des potentiometres
// Ajout de fonctionnalités de modulation par entrée CV
// Sauvegarde et chargement de presets dans l'EEPROM
// Edition d'une forme d'onde 16 bits utilisateurs
// Frederic ABIGNOLI 2015/02/06
//---------------------------------------------------------------
// 2015/02/06 Création du projet
// 2015/03/01 Implémentation du midi
// 2015/03/05 gestion des presets
//---------------------------------------------------------------
// A implementer
// - gestion forme onde utilisateur
// - gestion assignation entrees CV
//		CV1			CV2		CV3
//		OSC Freq	LFO Freq	LFO Depth
//		OSC Note	ENV Freq	ENV Depth
//					LFO Width	OSC Width
// - leds ???
//---------------------------------------------------------------

// POUR LA VIDEO
// Même moteur audio que le programme de MATT
// Remplacement des pot par des encodeurs
// Ajout d'un LCD pour editer les parametres
// Ajout sauvegarde / chargement de preset
// Ajout edition wave utilisateur




#include <MemoryFree.h>

//------------------------------------------------------
// Bibliotheque pour gerer lecture et ecriture en EEPROM
//------------------------------------------------------
#include <EEPROM.h>

#define AUDIOON	PORTB |= (1<<0)
#define AUDIOOFF	PORTB &= ~(1<<0)

//----------------------------------------------
// Bibliotheque et definitions pour gerer le LCD
//----------------------------------------------
#include <Wire.h>
#include <LiquidCrystal_I2C.h>
#if defined(ARDUINO) && ARDUINO >= 100
#define printByte(args)  write(args);
#else
#define printByte(args)  print(args,BYTE);
#endif
#include "OneBitSynth.h"
LiquidCrystal_I2C lcd(0x20,20,4);  							// sLCD adress 0x20, 16 car, 2 ligne
uint8_t symbUp[8]  = {0x4,0xe,0x15,0x4,0x4,0x4,0x4};    	//car fleche up
uint8_t symbDown[8]  = {0x4,0x4,0x4,0x4,0x15,0xe,0x4};  	//car fleche down
uint8_t symbTriUp[8]  = {0x4,0xe,0x1f,0x0,0x0,0x0,0x0};    	//car fleche up
uint8_t symbTriDown[8]  = {0x0,0x0,0x0,0x0,0x1f,0xe,0x4};  	//car fleche down
uint8_t symbTriRight[8]  = {0x8,0xc,0xe,0xf,0xe,0xc,0xc8};  //car triangle right
uint8_t symbTriLeft[8]  = {0x2,0x6,0xe,0x1e,0xe,0x6,0x2};  	//car fleche down
uint16_t RaffParamLcd[5] = {0,0,0,0,0};						// Parametre lcd a rafraichir
#define CAR_RIGHT 126   // Numero car fleche vers la droite 
#define CAR_LEFT  127   // Numero car fleche vers la gauche
#define CAR_UP    0     // Numero car fleche vers le haut
#define CAR_DOWN  1     // Numero car fleche vers le bas
#define CAR_TRIUP    2     // Numero car triangle vers le haut
#define CAR_TRIDOWN  3     // Numero car triangle vers le bas
#define CAR_TRIRIGHT  4     // Numero car triangle vers la droite
#define CAR_TRILEFT  5     // Numero car triangle vers la gauche

#define CAR_SEP   124   // Trait vertical de séparation
#define CAR_PLAIN 255   // Caractere plein


// ENCODEURS ET BOUTONS
//---------------------
int pinA[] = {5,7};               	// Pins A des encodeurs
int pinB[] = {4,6};               	// Pins B des encodeurs
unsigned int tcnt2;               	// Counter encoder 
int lastEncoded[] = {0,0}; // Dernieres valeur lue encodeurs
int counter[] = {0,0};     // Valeur courante des encodeurs
byte nencRead=0;			// numero encodeur actif pour la lecture dans ISR
long temps=0;
long maxtemps=0;
int bcounter[] = {0,0};             // valeurs encodeurs divisees par 4
int lcounter[] = {0,0};             // valeurs précédentes des encodeurs
byte valbouton;                     // Numero du bouton appuye
byte lastBoutonNav=0;       		// Stockage du dernier etat dun bouton
byte valencodeurs;          		// numero encodeur et sens
#define B_ENC1   1     				// Numero bouton encodeur 1
#define B_ENC2   2     				// Numero bouton encodeur 2
#define B_MAINT  9					// Bouton encodeur maintenu appuyé
#define ENC1DOWN  1     			// Encodeur 1 down
#define ENC2DOWN  2     			// Encodeur 2 down
#define ENC1UP    11    			// Encodeur 1 up
#define ENC2UP    12    			// Encodeur 2 up
byte nummenu=0;						// numero de menu courant
byte posmenu=0;						// position courante dans le menu courant
byte changemenu=0;					// flag defilement des menus avec encodeurs
#define MENUOSC		0				
#define MENULFO		1
#define MENUENV		2
#define MENUREFRESH	3
#define MINMENU1	0		// numero minimum de menu  encodeur 1
#define MAXMENU1	4		// numero maximum de menu  encodeur 1
#define MINMENU2	10		// numero minimum de menu  encodeur 2
#define MAXMENU2	11		// numero maximum de menu  encodeur 2
//byte refreshMenu=1;			// flag rafraichissement menu
char BuffAff[24];  		// buffer affichage pour PROGMEM entre autre
// Variable et constantes du synthe
volatile uint8_t wavenum = 6; 	// oscillator waveform
uint16_t pitch = 100;   		// set pitch
uint16_t freqvals[128] PROGMEM =
{
// Values for 16Mhz clocks
65535, 65535, 65535, 65535, 65535, 65535, 65535, 65535,
65535, 65535, 65535, 64792, 61155, 57723, 54483, 51425,
48539, 45814, 43243, 40816, 38525, 36363, 34322, 32395,
30577, 28861, 27241, 25712, 24269, 22907, 21621, 20407,
19262, 18181, 17160, 16197, 15288, 14430, 13620, 12855,
12134, 11453, 10810, 10203, 9630, 9090, 8580, 8098,
7644, 7214, 6809, 6427, 6066, 5726, 5404, 5101,
4815, 4544, 4289, 4049, 3821, 3607, 3404, 3213,
3033, 2862, 2702, 2550, 2407, 2272, 2144, 2024,
1910, 1803, 1702, 1606, 1516, 1431, 1350, 1275,
1203, 1135, 1072, 1011, 955, 901, 850, 803,
757, 715, 675, 637, 601, 567, 535, 505,
477, 450, 425, 401, 378, 357, 337, 318,
300, 283, 267, 252, 238, 224, 212, 200,
189, 178, 168, 168, 149, 141, 133, 126,
118, 112, 105, 99, 94, 88, 83, 79
};


uint16_t lfofreq = 0;   		// lfo period
uint16_t lfodepth = 0;  		// lfo wave amplitude
uint8_t envfreq = 0;    		// envelope period
uint8_t envwidth = 0;   		// envelope duty cycle
uint8_t lfowavenum = 0; 		// lfo waveform
static uint16_t waveforms[9] =
{
  0b1100000000000000,	// 49152
  0b1111111100000000,	// 65280
  0b1110111111101111,	// 61423
  0b1011011011011011,	// 46811
  0b0010100001110110,	// 10358
  0b1010101011010101,	// 43733
  0b1010101010101010,	// 43690
  0b0000000000000000, 	// noise
  0b0000000000000000  	// dummy (user)
};
// forme onde utilisateur
uint16_t UserWaveform = 0b1011100010111000;		// Forme onde 16 bits
byte PosUserWaveform=1;			// Position du bit a editer 1=bit 15 et 16= bit 0
#define MINPOSWAVEFORMUSER 1	// Position mini pour edition forme onde 16 bits
#define MAXPOSWAVEFORMUSER 16	// Position maxi pour edition forme onde 16 bits
#define USERWAVEFORMNUMBER 8	// Numero de la forme d'onde user
#define MAXWAVESOSC	8
#define MAXWAVESOSC	8
#define MAXWAVESLFO	7
#define MAXPITCHOSC	8192
#define	MAXLFOFREQ	255
#define MAXLFODEPTH	255
#define	MAXENVFREQ	255
#define MAXENVWIDTH	255
#define MIN_REFRESH_RATE_LFO 1
#define MIN_REFRESH_RATE_ENV 2
#define MAX_REFRESH_RATE_LFO 50
#define MAX_REFRESH_RATE_ENV 50
#define MIN_PITCH_INTERRUPT 30 	// Valeur OCR1A mini pour eviter plantage
#define CTRLLFOWAVE		110
#define CTRLLFOFREQ		111
#define CTRLLFODEPTH	112

int ttt;
// state
uint16_t outpitch = 0;  		// OCR1A value
uint8_t lfotimer = 0;   		// lfo value, counts from 0 to lfofreq
uint16_t lfoval = 0;    		// amount to add to output pitch
uint16_t lfodelta = 0;
uint8_t envtimer = 0;   		// envelope value, counts from 0 to envfreq
volatile uint8_t envval = 1; 	// high or low; ANDed with output wave
volatile uint16_t waveform;
long lastlfocnt=0;				// Compteur pour recalcul lfo
uint8_t RefreshRateLfo = 0; 	// Rafraichissement  et recalcul en ms du lfo
long lastenvcnt=0;				// Compteur pour Rafraichissement et recalcul enveloppe
uint8_t RefreshRateEnv= 0; 		// Rafraichissement  et recalcul en ms du lfo
#include <MIDI.h>
// Gestion du midi
enum { 	MIDI_OTHER, 
		MIDI_NOTE_OFF, 
		MIDI_NOTE_ON, 
		MIDI_CONTROL_CHANGE,
		MIDI_PROGRAM_CHANGE,
		MIDI_PITCH_BEND
};
uint8_t CountNote;     // Nombre de note appuyees sur le clavier a un instant "T"
volatile uint8_t usemidi = 0;	// enable midi or manual mode
volatile uint8_t noteon = 0;	// Etat de la sortie son

int MidiChannel=1;
// Destination des CV
uint8_t cv1dest = 0;    		// destination cv1
uint8_t cv2dest = 0;    		// destination cv2
uint8_t cv3dest = 0;    		// destination cv3



// Gestion des presets
#define MAXPRESETS	16			// Nombre de preset
#define PRESETSIZE	24			// taille d'un preset
uint8_t NumPresetLoad=1;		// Numero de preset a charger
uint8_t NumPresetSave=1;		// Numero de preset a sauvegarder
uint8_t initEEPROM = 0;			// flag initialisation de l'eeprom
byte myByte1,myByte2;			// 2 octets utilisateurs
int myaddress=0;				// pointeur pour EEPROM
uint16_t RaffCount=0;
#define DELAYRAFF	200
//---------------------------------
// SETUP DES VARIABLES
// Preparation des PINS
// Initialisation des interruptions
//---------------------------------
void setup() {
	noteon=0;						// Couper le son
	MidiChannel=1;
	RefreshRateLfo=10;				// Initialisation Refresh Lfo
	RefreshRateEnv=10;				// Initialisation Refresh Env
	//--------------
	// Init wave ISR
	//--------------
	cli();							// disable interrupts
	TCCR1A = 0;						// set entire TCCR1A register to 0
	TCCR1B = 0;						// same for TCCR1B
	TCNT1 = 0;						// initialize counter value to 0
	OCR1A = 0;						// reset OCR1A
	TCCR1B |= (1 << WGM12);			// turn on CTC mode
	TCCR1B |= (1 << CS11);			// Set CS11 bit for 8 prescaler
	TIMSK1 |= (1 << OCIE1A);		// enable timer compare interrupt
	waveform = waveforms[wavenum];	// Charger le registre de forme d'onde
	update_pitch();					// Updater le pitch oscillateur
	sei(); 							// enable interrupts		

	pinMode(8, OUTPUT);				// Sortie audio
	pinMode(A0, INPUT);				// Entres pour les interrupteurs encodeurs
	pinMode(A1, INPUT);				// Entree CV1
	pinMode(A2, INPUT);				// Entree CV2
	pinMode(A3, INPUT);				// Entree CV3
	//------------------
	// Init encoders ISR
	//------------------
	encodeursSetup();

	lcd.init();                     			// init du lcd 
	lcd.backlight();                			// Retroeclairage
	lcd.createChar(0, symbUp);      			// Creation fleche UP 
	lcd.createChar(1, symbDown);    			// Creation fleche DOWN
	lcd.createChar(2, symbTriUp);      			// Creation triangle UP
	lcd.createChar(3, symbTriDown);    			// Creation triangle DOWN
	lcd.createChar(4, symbTriRight);   			// Creation triangle DOWN
	lcd.createChar(5, symbTriLeft);    			// Creation triangle DOWN
	
	
	
	// Affichage du titre
	lcd.home();                     			
	lcd.print(F("********************"));
	lcd.setCursor(0,1);
	lcd.print(F("*** OneBit SYNTH ***"));
	lcd.setCursor(0,2);
	lcd.print(F("*   Version 0.96   *"));
	lcd.setCursor(0,3);
	lcd.print(F("********************"));
	delay(1000);							// Attente 1 sec, permet d'appuyer sur bouton 2
	initEEPROM=0;							// Reset flag initialisation eeprom
	valbouton=readButtons();				// Lire les interrupteurs des encodeurs
	if (valbouton==B_ENC2)					// Si interrupteurs 2 appuye
		initEEPROM=1;						// Valider la possible initialisation de l'EEPROM
	lcd.clear();
	// Affiche memoire programme libre
	lcd.print(F("Free Memory: "));
	lcd.print(freeMemory(),DEC);
	delay(1000);
	valbouton=readButtons();				// Regarder si le bouton est toujours enfoncer
	if (valbouton==B_MAINT && initEEPROM)	// Si oui et que initEEPROM a deja ete valide
		InitialisationEEPROM();				// Lancer l'initialisation de l'EEPROM
	//---------------------
	// Init serial midi ISR
	//---------------------
	InitMenu();								// Forcer affichage menu principal
	lastlfocnt=millis();					// RAZ heure maj lfo
	lastenvcnt=millis();					// RAZ heure maj enveloppe
	noteon=1;
	// Initialisation du MIDI
	//-----------------------
	MIDI.begin(MidiChannel);              // demarre le midi en mode OMNI
	CountNote=0;                                // Compteur de note actives
	
	// Initialisation Handler Midi
	//----------------------------
	MIDI.setHandleNoteOn(MyHandleNoteOnCV);       	// handler note on 
	MIDI.setHandleNoteOff(MyHandleNoteOffCV);     	// handler note off
	MIDI.setHandleControlChange(MyHandleCtrlCV);  	// handler CC
}



//--------------------
// PROGRAMME PRINCIPAL
//--------------------
//--------------------
//--------------------
void loop() {
		MIDI.read();
		ReadCvGate();
		RaffLcd();
		RefreshParamMenu();
		update_synth_params();
		update_lfo();
		update_env();
		update_pitch();
		scan_encodeurs();
		delayMicroseconds(100);
}
