//------------------------------------------
// Fonctions et procedures du synthetiseur
// tire du programme 1-Bit AVR Synthesizer
//------------------------------------------
// http://www.msarnoff.org/1bitsynth/
// 1-Bit AVR Synthesizer
// By Matt Sarnoff (msarnoff.org/1bitsynth)
// October 2009-October 2010
//------------------------------------------

//------------------------
// Interruption du timer 1
// Gere la sortie audio
//------------------------
ISR(TIMER1_COMPA_vect) {
uint8_t shiftout;	// Etat sortie audio
	if (wavenum != 7) {
		shiftout = waveform & 1;
		waveform >>= 1;
		if (shiftout)
			waveform |= (1 << 15);
	}
	else {
		shiftout = lfsr_rand() & 1;
	}

	// apply envelope
	shiftout &= envval;
	 
	// mute if note off
	shiftout &= noteon;

  // update audio pin
	if (shiftout)
		AUDIOON;
	else
		AUDIOOFF;
}

// faster noise generator than rand()
uint16_t lfsr_rand()
{
  static uint16_t lfsr = 0xACE1u;
  lfsr = (lfsr >> 1) ^ (-(lfsr & 1u) & 0xB400u);
  return lfsr;
}


//---------------------------------------
// Mise a jour du pitch
// En fonction des reglages ou de la note
// tient compte de la variation du lfo
//---------------------------------------
void update_pitch()
{

	uint16_t newpitch = (usemidi) ? pitch-(lfoval*4U)
                                  : 200U + ((pitch-lfoval)*4U);
                          
	if (newpitch<MIN_PITCH_INTERRUPT)
		return;
	if (newpitch != outpitch)
	{
		outpitch = newpitch;
	
		if (TCNT1 > outpitch)
			TCNT1 = 0;
		OCR1A = outpitch;
   }
}

//----------------------------
// Mise a jour de letat du LFO
//----------------------------
void update_lfo()
{
	if (!lfodepth || !lfofreq) {
		lfoval=0;
		return;
	}
	// Verifie si le lfo doit être mise a jour
	// si le temps "RefreshRateLfo" est depasse
	 if (millis() < lastlfocnt+RefreshRateLfo) 
		  return;
	lastlfocnt=millis();
	
	//-------------------------------
	// calcul de lfoval
	// en fonction de sa forme d'onde
	//-------------------------------
	switch (lfowavenum) {
		case 0: // triangle
			lfoval = (lfotimer*lfodelta) >> 8;
			if (lfotimer >= lfofreq/2)
				lfoval = lfodepth - lfoval;
			break;
		case 1: // sawtooth up
			lfoval = (lfotimer*lfodelta) >> 8;
			break;
		case 2: // sawtooth down
			lfoval = lfodepth - ((lfotimer*lfodelta) >> 8);
			break;
		case 3: // square
			lfoval = (lfotimer >= lfofreq/2) ? lfodepth : 0;
			break;
		case 4: // half square
			lfoval = (lfotimer < lfofreq/4) ? lfodepth : 0;
			break;
		case 5: // half sawtooth up
			if (lfotimer < lfofreq/2)
				lfoval = (lfotimer*lfodelta) >> 7;
			else
				lfoval = 0;
			break;
		case 6: // half sawtooth down
			if (lfotimer < lfofreq/2)
				lfoval = lfodepth - ((lfotimer*lfodelta) >> 7);
			else
				lfoval = 0;
			break;
		case 7: // random
			if (lfotimer == 0)
				lfoval = lfsr_rand() % lfodepth;
			break;
	}
	lfotimer++;
	if (lfotimer >= lfofreq)
		lfotimer = 0;
}


// update envelope state
void update_env() {

	// Verifie si le lfo doit être mise a jour
	// si le temps "RefreshRateLfo" est depasse
	if (millis() < lastenvcnt+RefreshRateEnv) 
		return;
	lastenvcnt=millis();
	// increment envelope timer and wrap around
	envtimer++;
	if (envtimer >= envfreq) {
		envtimer = 0;
	}

	// determine output value
	envval = (envtimer <= envwidth);
}
// update parameters based on knob values
void update_synth_params() {
	// if (!usemidi)
		// pitch = pitchButton;
	lfodelta = (lfofreq) ? (lfodepth*256U) / lfofreq : 0;
}

//---------------------------------------
// recharger le buffer waveform
// avec la forme d'onde user UserWaveform
// uniquement si la forme d'onde en cours
// est celle de l'utilisateur
//---------------------------------------
void ReloadUserWaveform(void) {
	if (wavenum==USERWAVEFORMNUMBER) {			// Si la forme d'onde en court est bien celle de l'utilisateur
		if (!usemidi) 							// Si nous ne sommes pas en mode midi
			noteon=0;							// couper le son
		waveform=UserWaveform;					// rafraichir le buffer son avec la forme onde user
		delay(10);								// pour limiter le craquement
		if (!usemidi) 							// Si nous ne sommes pas en mode midi
			noteon=1;								// retablir le son
	}
}
